@@ -40,9 +40,9 @@
 #include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 
-#include "e-dbhash.h"
-#include "e-db3-utils.h"
-#include "libedataserver/e-data-server-util.h"
+#include "libedataserver/e-dbhash.h"
+#include "libedataserver/e-db3-utils.h"
+#include "libedataserver/e-util.h"
 
 #include "libebook/e-contact.h"
 
@@ -53,7 +53,7 @@
 
 #include "e-book-backend-file.h"
 
-#define d(x)
+#define d(x) x
 
 #define CHANGES_DB_SUFFIX ".changes.db"
 
@@ -85,24 +85,6 @@ static struct {
 	DB_ENV *env;
 } global_env;
 
-static EBookBackendSyncStatus
-db_error_to_status (const int db_error)
-{
-	switch (db_error) {
-	case 0:
-		return GNOME_Evolution_Addressbook_Success;
-	case DB_NOTFOUND:
-		return GNOME_Evolution_Addressbook_ContactNotFound;
-	case EACCES:
-		return GNOME_Evolution_Addressbook_PermissionDenied;
-	case ENOMEM:
-	case ENOSPC:
-		return GNOME_Evolution_Addressbook_NoSpace;
-	default:
-		return GNOME_Evolution_Addressbook_OtherError;
-	}
-}
-
 static void
 string_to_dbt(const char *str, DBT *dbt)
 {
@@ -133,7 +115,7 @@ build_summary (EBookBackendFilePrivate *bfpriv)
 	db_error = db->cursor (db, NULL, &dbc, 0);
 
 	if (db_error != 0) {
-		g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
+		g_warning ("build_summary: error building list\n");
 		return;
 	}
 
@@ -183,33 +165,29 @@ set_revision (EContact *contact)
 	
 }
 
-static EBookBackendSyncStatus
+static EContact *
 do_create(EBookBackendFile  *bf,
-	  const char      *vcard_req,
-	  EContact **contact)
+	  const char      *vcard_req)
 {
 	DB             *db = bf->priv->file_db;
 	DBT            id_dbt, vcard_dbt;
 	int            db_error;
 	char           *id;
+	EContact       *contact;
 	char           *vcard;
 	const char *rev;
 	
-	g_assert (bf);
-	g_assert (vcard_req);
-	g_assert (contact);
-
 	id = e_book_backend_file_create_unique_id ();
 
 	string_to_dbt (id, &id_dbt);
 
-	*contact = e_contact_new_from_vcard (vcard_req);
-	e_contact_set (*contact, E_CONTACT_UID, id);
-	rev = e_contact_get_const (*contact,  E_CONTACT_REV);
+	contact = e_contact_new_from_vcard (vcard_req);
+	e_contact_set (contact, E_CONTACT_UID, id);
+	rev = e_contact_get_const (contact,  E_CONTACT_REV);
 	if (!(rev && *rev))
-		set_revision (*contact);
+		set_revision (contact);
 
-	vcard = e_vcard_to_string (E_VCARD (*contact), EVC_FORMAT_VCARD_30);
+	vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
 
 	string_to_dbt (vcard, &vcard_dbt);
 
@@ -219,17 +197,17 @@ do_create(EBookBackendFile  *bf,
 
 	if (0 == db_error) {
 		db_error = db->sync (db, 0);
-		if (db_error != 0) {
-			g_warning ("db->sync failed with %s", db_strerror (db_error));
-		}
-	} else {
-		g_warning (G_STRLOC ": db->put failed with %s", db_strerror (db_error));
-		g_object_unref (*contact);
-		*contact = NULL;
+		if (db_error != 0)
+			g_warning ("db->sync failed with %d", db_error);
+	}
+	else {
+		g_warning ("db->put failed with %d", db_error);
+		g_object_unref (contact);
+		contact = NULL;
 	}
 
 	g_free (id);
-	return db_error_to_status (db_error);
+	return contact;
 }
 
 static EBookBackendSyncStatus
@@ -239,14 +217,18 @@ e_book_backend_file_create_contact (EBookBackendSync *backend,
 				    const char *vcard,
 				    EContact **contact)
 {
-	EBookBackendSyncStatus status;
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 
-	status = do_create (bf, vcard, contact);
-	if (status == GNOME_Evolution_Addressbook_Success) {
+	*contact = do_create (bf, vcard);
+	if (*contact) {
 		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
+		return GNOME_Evolution_Addressbook_Success;
+	}
+	else {
+		/* XXX need a different call status for this case, i
+                   think */
+		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
-	return status;
 }
 
 static EBookBackendSyncStatus
@@ -272,8 +254,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 
 		db_error = db->del (db, NULL, &id_dbt, 0);
 		if (0 != db_error) {
-			g_warning (G_STRLOC ": db->del failed with %s", db_strerror (db_error));
-			rv = db_error_to_status (db_error);
+			rv = GNOME_Evolution_Addressbook_ContactNotFound;
 			continue;
 		}
 
@@ -284,7 +265,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 	if (removed_cards) {
 		db_error = db->sync (db, 0);
 		if (db_error != 0)
-			g_warning (G_STRLOC ": db->sync failed with %s", db_strerror (db_error));
+			g_warning ("db->sync failed.\n");
 	}
 
 	*ids = removed_cards;
@@ -297,59 +278,6 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 	return rv;
 }
 
-static EBookBackendSyncStatus
-modify_contact (EBookBackendFile *bf, const char *vcard, EContact **contact)
-{
-	DB *db;
-	int db_error;
-	char *vcard_with_rev;
-	const char *id, *dbt_id;
-	DBT id_dbt, vcard_dbt;
-
-	g_assert (bf);
-	g_assert (vcard);
-	g_assert (contact);
-	g_assert (*contact == NULL);
-
-	db = bf->priv->file_db;
-
-	*contact = e_contact_new_from_vcard (vcard);
-	id = e_contact_get_const (*contact, E_CONTACT_UID);
-
-	if (id == NULL)
-		return GNOME_Evolution_Addressbook_OtherError;
-	
-	/* update the revision (modified time of contact) */
-	set_revision (*contact);
-	vcard_with_rev = e_vcard_to_string (E_VCARD (*contact), EVC_FORMAT_VCARD_30);
-
-	/*
-	 * This is disgusting, but for a time cards were added with IDs that are
-	 * no longer used (they contained both the URI and the ID). If we
-	 * recognize it as a URI (file:///...) trim off everything before the
-	 * last '/', and use that as the ID.
-	 */
-	if (!strncmp (id, "file:///", strlen ("file:///")))
-		dbt_id = strrchr (id, '/') + 1;
-	else
-		dbt_id = id;
-	
-	string_to_dbt (dbt_id, &id_dbt);
-	string_to_dbt (vcard_with_rev, &vcard_dbt);
-
-	db_error = db->put (db, NULL, &id_dbt, &vcard_dbt, 0);
-
-	g_free (vcard_with_rev);
-	
-	if (db_error == 0) {
-		return GNOME_Evolution_Addressbook_Success;
-	} else {
-		g_warning (G_STRLOC ": db->put failed with %s", db_strerror (db_error));
-		return db_error_to_status (db_error);
-	}
-
-}
-
 static EBookBackendSyncStatus
 e_book_backend_file_modify_contact (EBookBackendSync *backend,
 				    EDataBook *book,
@@ -357,73 +285,64 @@ e_book_backend_file_modify_contact (EBookBackendSync *backend,
 				    const char *vcard,
 				    EContact **contact)
 {
-	EBookBackendSyncStatus status;
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	DB             *db = bf->priv->file_db;
+	DBT            id_dbt, vcard_dbt;
 	int            db_error;
+	char          *id, *lookup_id;
+	char          *vcard_with_rev;
 
-	status = modify_contact (bf, vcard, contact);
+	*contact = e_contact_new_from_vcard (vcard);
+	id = e_contact_get(*contact, E_CONTACT_UID);
+
+	if (id == NULL)
+		return GNOME_Evolution_Addressbook_OtherError;
+
+	/* This is disgusting, but for a time cards were added with
+           ID's that are no longer used (they contained both the uri
+           and the id.) If we recognize it as a uri (file:///...) trim
+           off everything before the last '/', and use that as the
+           id.*/
+	if (!strncmp (id, "file:///", strlen ("file:///"))) {
+		lookup_id = strrchr (id, '/') + 1;
+	}
+	else
+		lookup_id = id;
+
+	string_to_dbt (lookup_id, &id_dbt);	
+	memset (&vcard_dbt, 0, sizeof (vcard_dbt));
+	vcard_dbt.flags = DB_DBT_MALLOC;
+
+	/* get the old ecard - the one that's presently in the db */
+	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
+	if (0 != db_error)
+		return GNOME_Evolution_Addressbook_ContactNotFound;
+
+	free (vcard_dbt.data);
+
+	/* update the revisio (modified time of contact) */
+	set_revision (*contact);
+	vcard_with_rev = e_vcard_to_string (E_VCARD (*contact), EVC_FORMAT_VCARD_30);
 	
-	if (status == GNOME_Evolution_Addressbook_Success) {
+	string_to_dbt (vcard_with_rev, &vcard_dbt);	
+
+	db_error = db->put (db, NULL, &id_dbt, &vcard_dbt, 0);
+
+	if (0 == db_error) {
 		db_error = db->sync (db, 0);
-		if (db_error == 0) {
-			e_book_backend_summary_remove_contact (bf->priv->summary,
-							       e_contact_get_const (*contact, E_CONTACT_UID));
-			e_book_backend_summary_add_contact (bf->priv->summary, *contact);
-			
-			return GNOME_Evolution_Addressbook_Success;
-		} else {
-			g_warning (G_STRLOC ": db->sync failed with %s", db_strerror (db_error));
-			return db_error_to_status (db_error);
-		}
-	} else {
-		return status;
+		if (db_error != 0)
+			g_warning ("db->sync failed.\n");
+
+		e_book_backend_summary_remove_contact (bf->priv->summary, id);
+		e_book_backend_summary_add_contact (bf->priv->summary, *contact);
 	}
-}
+	g_free (id);
+	g_free (vcard_with_rev);
 
-static EBookBackendSyncStatus
-e_book_backend_file_modify_contacts (EBookBackendSync *backend,
-				    EDataBook *book,
-				    guint32 opid,
-				    const char **vcards,
-				    GList **contacts)
-{
-	EBookBackendFile *bf;
-	EBookBackendSyncStatus status = GNOME_Evolution_Addressbook_OtherError;
-	EContact *contact;
-	DB *db;
-	int db_error;
-
-	bf = E_BOOK_BACKEND_FILE (backend);
-	db = bf->priv->file_db;
-	
-	/* Commit each of the new contacts, aborting if there was an error.
-	   Really this should be in a transaction... */
-	for (; *vcards; vcards++) {
-		contact = NULL;
-
-		/* Commit the contact */
-		status = modify_contact (bf, *vcards, &contact);
-		if (status != GNOME_Evolution_Addressbook_Success)
-			goto done;
-
-		/* Update the summary */
-		e_book_backend_summary_remove_contact (bf->priv->summary,
-						       e_contact_get_const (contact, E_CONTACT_UID));
-		e_book_backend_summary_add_contact (bf->priv->summary, contact);
-
-		/* Pass the contact back to the server for view updates */
-		*contacts = g_list_prepend (*contacts, contact);
-	}
-
-	/* Sync the database */
-	db_error = db->sync (db, 0);
-	if (db_error != 0) {
-		return db_error_to_status (db_error);
-	}
-
- done:
-	return status;
+	if (0 == db_error)
+		return GNOME_Evolution_Addressbook_Success;
+	else
+		return GNOME_Evolution_Addressbook_ContactNotFound;
 }
 
 static EBookBackendSyncStatus
@@ -438,7 +357,7 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 	DBT             id_dbt, vcard_dbt;
 	int             db_error = 0;
 
-	bf = E_BOOK_BACKEND_FILE (backend);
+	bf = E_BOOK_BACKEND_FILE (e_data_book_get_backend (book));
 	db = bf->priv->file_db;
 
 	string_to_dbt (id, &id_dbt);
@@ -448,10 +367,11 @@ e_book_backend_file_get_contact (EBookBackendSync *backend,
 	db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 	if (db_error == 0) {
-		*vcard = vcard_dbt.data;
+		*vcard = g_strdup (vcard_dbt.data);
+		printf ("backend %s\n", *vcard);
+		free (vcard_dbt.data);
 		return GNOME_Evolution_Addressbook_Success;
 	} else {
-		g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
 		*vcard = g_strdup ("");
 		return GNOME_Evolution_Addressbook_ContactNotFound;
 	}
@@ -490,15 +410,15 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 			vcard_dbt.flags = DB_DBT_MALLOC;
 
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
-			if (db_error == 0) {
-				contact_list = g_list_prepend (contact_list, vcard_dbt.data);
-			} else {
-				g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
-				status = db_error_to_status (db_error);
+			if (db_error == 0)
+				contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
+			else { 
+				status = GNOME_Evolution_Addressbook_OtherError ;
 				break;
 			}
 		}
 		g_ptr_array_free (ids, TRUE);
+		
 	} else {
 		search_needed = TRUE;
 		if (!strcmp (search, "(contains \"x-evolution-any-field\" \"\")"))
@@ -507,19 +427,17 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		card_sexp = e_book_backend_sexp_new (search);
 		if (!card_sexp) {
 			/* XXX this needs to be an invalid query error of some sort*/
-			return GNOME_Evolution_Addressbook_OtherError;
+			return GNOME_Evolution_Addressbook_ContactNotFound;
 		}
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 
 		if (db_error != 0) {
-			g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
 			/* XXX this needs to be some CouldNotOpen error */
-			return db_error_to_status (db_error);
+			return GNOME_Evolution_Addressbook_ContactNotFound;
 		}
 
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
-		vcard_dbt.flags = DB_DBT_MALLOC;
 		memset (&id_dbt, 0, sizeof (id_dbt));
 		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
@@ -530,7 +448,7 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 			    || strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
 
 				if ((!search_needed) || (card_sexp != NULL && e_book_backend_sexp_match_vcard  (card_sexp, vcard_dbt.data))) {
-					contact_list = g_list_prepend (contact_list, vcard_dbt.data);
+					contact_list = g_list_append (contact_list, g_strdup (vcard_dbt.data));
 				}
 			}
 
@@ -539,16 +457,13 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 		}
 		g_object_unref (card_sexp);
 		
-		if (db_error == DB_NOTFOUND) {
-			status = GNOME_Evolution_Addressbook_Success;
-		} else {
-			g_warning (G_STRLOC ": dbc->c_get failed with %s", db_strerror (db_error));
-			status = db_error_to_status (db_error);
-		}
+		status = db_error != DB_NOTFOUND
+			? GNOME_Evolution_Addressbook_OtherError
+			: GNOME_Evolution_Addressbook_Success;
 
 		db_error = dbc->c_close(dbc);
 		if (db_error != 0) {
-			g_warning (G_STRLOC ": dbc->c_close failed with %s", db_strerror (db_error));
+			g_warning("Could not close cursor: %d", db_error);
 		}
 	}
 
@@ -599,41 +514,28 @@ get_closure (EDataBookView *book_view)
 static gpointer
 book_view_thread (gpointer data)
 {
-	EDataBookView *book_view;
-	FileBackendSearchClosure *closure;
-	EBookBackendFile *bf;
+	EDataBookView *book_view = data;
+	FileBackendSearchClosure *closure = get_closure (book_view);
+	EBookBackendFile *bf = closure->bf;
 	const char *query;
 	DB  *db;
 	DBT id_dbt, vcard_dbt;
 	int db_error;
-	gboolean stopped = FALSE, allcontacts;
-
-	g_return_val_if_fail (E_IS_DATA_BOOK_VIEW (data), NULL);
-
-	book_view = data;
-	closure = get_closure (book_view);
-	if (!closure) {
-		g_warning (G_STRLOC ": NULL closure in book view thread");
-		return NULL;
-	}
-	bf = closure->bf;
+	gboolean stopped = FALSE;
 
 	d(printf ("starting initial population of book view\n"));
 
 	/* ref the book view because it'll be removed and unrefed
 	   when/if it's stopped */
-	g_object_ref (book_view);
+	bonobo_object_ref (book_view);
 	
 	db = bf->priv->file_db;
 	query = e_data_book_view_get_card_query (book_view);
 
-	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")")) {
+	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")"))
 		e_data_book_view_notify_status_message (book_view, _("Loading..."));
-		allcontacts = TRUE;
-	} else {
+	else
 		e_data_book_view_notify_status_message (book_view, _("Searching..."));
-		allcontacts = FALSE;
-	}
 
 	d(printf ("signalling parent thread\n"));
 	g_mutex_lock (closure->mutex);
@@ -661,10 +563,16 @@ book_view_thread (gpointer data)
 			db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
 
 			if (db_error == 0) {
-				e_data_book_view_notify_update_prefiltered_vcard (book_view, id, vcard_dbt.data);
+				EContact *contact = create_contact (id_dbt.data, vcard_dbt.data);
+				/* notify_update will check if it matches for us */
+				e_data_book_view_notify_update (book_view, contact);
+				
+				g_object_unref (contact);
+
+				free (vcard_dbt.data);
 			}
 			else {
-				g_warning (G_STRLOC ": db->get failed with %s", db_strerror (db_error));
+				g_warning ("db->get returned %d", db_error);
 			}
 		}
 
@@ -676,7 +584,6 @@ book_view_thread (gpointer data)
 
 		memset (&id_dbt, 0, sizeof (id_dbt));
 		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
-		vcard_dbt.flags = DB_DBT_MALLOC;
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 		if (db_error == 0) {
@@ -693,24 +600,21 @@ book_view_thread (gpointer data)
 
 				/* don't include the version in the list of cards */
 				if (strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
-					if (allcontacts)
-						e_data_book_view_notify_update_prefiltered_vcard (book_view, id_dbt.data, vcard_dbt.data);
-					else
-						e_data_book_view_notify_update_vcard (book_view, vcard_dbt.data);
-				} else {
-					g_free (vcard_dbt.data);
+					EContact *contact = create_contact (id_dbt.data, vcard_dbt.data);
+					/* notify_update will check if it matches for us */
+					e_data_book_view_notify_update (book_view, contact);
+					g_object_unref (contact);
 				}
 
 				db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_NEXT);
 			}
 
 			dbc->c_close (dbc);
-			if (db_error && db_error != DB_NOTFOUND)
-				g_warning ("e_book_backend_file_search: error building list: %s",
-					   db_strerror (db_error));
+			if (db_error != DB_NOTFOUND)
+				g_warning ("e_book_backend_file_search: error building list\n");
 		}
 		else if (db_error == DB_RUNRECOVERY) {
-			g_warning ("e_book_backend_file_search: error getting the cursor for %s",
+			g_warning ("e_book_backend_file_search: error getting the cursor for %s\n",
 				   bf->priv->filename);
 			abort ();
 		}
@@ -722,7 +626,8 @@ book_view_thread (gpointer data)
 		e_data_book_view_notify_complete (book_view, GNOME_Evolution_Addressbook_Success);
 
 	/* unref the */
-	g_object_unref (book_view);
+	printf("book_view file uref \n");
+	bonobo_object_unref (book_view);
 
 	d(printf ("finished population of book view\n"));
 
@@ -733,17 +638,12 @@ static void
 e_book_backend_file_start_book_view (EBookBackend  *backend,
 				     EDataBookView *book_view)
 {
-	FileBackendSearchClosure *closure;
-
-	g_return_if_fail (E_IS_BOOK_BACKEND_FILE (backend));
-	g_return_if_fail (E_IS_DATA_BOOK_VIEW (book_view));
-
-	closure = init_closure (book_view, E_BOOK_BACKEND_FILE (backend));
+	FileBackendSearchClosure *closure = init_closure (book_view, E_BOOK_BACKEND_FILE (backend));
 
 	g_mutex_lock (closure->mutex);
 
 	d(printf ("starting book view thread\n"));
-	closure->thread = g_thread_create (book_view_thread, book_view, FALSE, NULL);
+	closure->thread = g_thread_create (book_view_thread, book_view, TRUE, NULL);
 
 	g_cond_wait (closure->cond, closure->mutex);
 
@@ -757,16 +657,17 @@ e_book_backend_file_stop_book_view (EBookBackend  *backend,
 				    EDataBookView *book_view)
 {
 	FileBackendSearchClosure *closure = get_closure (book_view);
+	gboolean need_join = FALSE;
 
 	d(printf ("stopping query\n"));
-	if (!closure) {
-		/* TODO: find out why this happens! */
-		g_warning ("NULL closure when stopping query");
-		return;
-	}
 	g_mutex_lock (closure->mutex);
+	if (!closure->stopped)
+		need_join = TRUE;
 	closure->stopped = TRUE;
 	g_mutex_unlock (closure->mutex);
+
+	if (need_join)
+		g_thread_join (closure->thread);
 }
 
 typedef struct {
@@ -811,7 +712,7 @@ e_book_backend_file_changes_foreach_key (const char *key, gpointer user_data)
 
 		g_object_unref (contact);
 		
-		g_free (vcard_dbt.data);
+		free (vcard_dbt.data);
 	}
 }
 
@@ -849,7 +750,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	db_error = db->cursor (db, NULL, &dbc, 0);
 
 	if (db_error != 0) {
-		g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
+		g_warning ("e_book_backend_file_changes: error building list\n");
 	} else {
 		db_error = dbc->c_get(dbc, &id_dbt, &vcard_dbt, DB_FIRST);
 
@@ -868,7 +769,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 				 */
 				contact = create_contact (id_dbt.data, vcard_dbt.data);
 
-#ifdef notyet
+#if notyet
 				g_object_set (card, "last_use", NULL, "use_score", 0.0, NULL);
 #endif
 				vcard_string = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
@@ -877,7 +778,6 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 				/* check what type of change has occurred, if any */
 				switch (e_dbhash_compare (ehash, id, vcard_string)) {
 				case E_DBHASH_STATUS_SAME:
-					g_free(vcard_string);
 					break;
 				case E_DBHASH_STATUS_NOT_FOUND:
 					ctx.add_cards = g_list_append (ctx.add_cards, vcard_string);
@@ -901,7 +801,7 @@ e_book_backend_file_get_changes (EBookBackendSync *backend,
 	if (db_error != DB_NOTFOUND) {
 		g_warning ("e_book_backend_file_changes: error building list\n");
 		*changes_out = NULL;
-		result = db_error_to_status (db_error);
+		result = GNOME_Evolution_Addressbook_OtherError;
 	}
 	else {
 		/* Update the hash and build our changes list */
@@ -1036,7 +936,7 @@ e_book_backend_file_upgrade_db (EBookBackendFile *bf, char *old_version)
 
 		db_error = db->cursor (db, NULL, &dbc, 0);
 		if (db_error != 0) {
-			g_warning (G_STRLOC ": db->cursor failed with %s", db_strerror (db_error));
+			g_warning ("unable to get cursor");
 			return FALSE;
 		}
 
@@ -1082,7 +982,7 @@ e_book_backend_file_upgrade_db (EBookBackendFile *bf, char *old_version)
 		}
 
 		if (card_failed) {
-			g_warning ("failed to update %d cards", card_failed);
+			g_warning ("failed to update %d cards\n", card_failed);
 			return FALSE;
 		}
 	}
@@ -1113,7 +1013,8 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 	db_error = db->get (db, NULL, &version_name_dbt, &version_dbt, 0);
 	if (db_error == 0) {
 		/* success */
-		version = version_dbt.data;
+		version = g_strdup (version_dbt.data);
+		free (version_dbt.data);
 	}
 	else {
 		/* key was not in file */
@@ -1133,11 +1034,7 @@ e_book_backend_file_maybe_upgrade_db (EBookBackendFile *bf)
 #endif
 
 static void
-#if DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3
-file_errcall (const DB_ENV *env, const char *buf1, const char *buf2)
-#else
 file_errcall (const char *buf1, char *buf2)
-#endif
 {
 	g_warning ("libdb error: %s", buf2);
 }
@@ -1166,10 +1063,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 	db_error = e_db3_utils_maybe_recover (filename);
 	if (db_error != 0) {
-		g_warning ("db recovery failed with %s", db_strerror (db_error));
+		g_warning ("db recovery failed with %d", db_error);
 		g_free (dirname);
 		g_free (filename);
-		return db_error_to_status (db_error);
+		return GNOME_Evolution_Addressbook_OtherError;
 	}
 
 	g_static_mutex_lock(&global_env_lock);
@@ -1179,30 +1076,25 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	} else {
 		db_error = db_env_create (&env, 0);
 		if (db_error != 0) {
-			g_warning ("db_env_create failed with %s", db_strerror (db_error));
+			g_warning ("db_env_create failed with %d", db_error);
 			g_static_mutex_unlock(&global_env_lock);
 			g_free (dirname);
 			g_free (filename);
-			return db_error_to_status (db_error);
+			return GNOME_Evolution_Addressbook_OtherError;
 		}
 
-		env->set_errcall (env, file_errcall);
-
-		/* Set the allocation routines to the non-aborting GLib functions */
-		env->set_alloc (env, (void *(*)(size_t))g_try_malloc, 
-				(void *(*)(void *, size_t))g_try_realloc,
-				g_free);
-
 		db_error = env->open (env, NULL, DB_CREATE | DB_INIT_MPOOL | DB_PRIVATE | DB_THREAD, 0);
 		if (db_error != 0) {
 			env->close(env, 0);
-			g_warning ("db_env_open failed with %s", db_strerror (db_error));
+			g_warning ("db_env_open failed with %d", db_error);
 			g_static_mutex_unlock(&global_env_lock);
 			g_free (dirname);
 			g_free (filename);
-			return db_error_to_status (db_error);
+			return GNOME_Evolution_Addressbook_OtherError;
 		}
 
+		env->set_errcall (env, file_errcall);
+
 		global_env.env = env;
 		global_env.ref_count = 1;
 	}
@@ -1212,10 +1104,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 	db_error = db_create (&db, env, 0);
 	if (db_error != 0) {
-		g_warning ("db_create failed with %s", db_strerror (db_error));
+		g_warning ("db_create failed with %d", db_error);
 		g_free (dirname);
 		g_free (filename);
-		return db_error_to_status (db_error);
+		return GNOME_Evolution_Addressbook_OtherError;
 	}
 
 	db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
@@ -1224,10 +1116,10 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		db_error = e_db3_utils_upgrade_format (filename);
 
 		if (db_error != 0) {
-			g_warning ("db format upgrade failed with %s", db_strerror (db_error));
+			g_warning ("db format upgrade failed with %d", db_error);
 			g_free (dirname);
 			g_free (filename);
-			return db_error_to_status (db_error);
+			return GNOME_Evolution_Addressbook_OtherError;
 		}
 
 		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_THREAD, 0666);
@@ -1238,15 +1130,6 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	if (db_error == 0) {
 		writable = TRUE;
 	} else {
-		db->close (db, 0);
-		
-        	db_error = db_create (&db, env, 0);
-        	if (db_error != 0) {
-                	g_warning ("db_create failed with %s", db_strerror (db_error));
-                	g_free (dirname);
-                	g_free (filename);
-                	return GNOME_Evolution_Addressbook_OtherError;
-        	}
 		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_RDONLY | DB_THREAD, 0666);
 
 		if (db_error != 0 && !only_if_exists) {
@@ -1254,41 +1137,28 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 			/* the database didn't exist, so we create the
 			   directory then the .db */
-			db->close (db, 0);
-			rv = g_mkdir_with_parents (dirname, 0777);
+			rv = e_util_mkdir_hier (dirname, 0777);
 			if (rv == -1 && errno != EEXIST) {
 				g_warning ("failed to make directory %s: %s", dirname, strerror (errno));
 				g_free (dirname);
 				g_free (filename);
 				if (errno == EACCES || errno == EPERM)
 					return GNOME_Evolution_Addressbook_PermissionDenied;
-				else if (errno == ENOSPC)
-					return GNOME_Evolution_Addressbook_NoSpace;
 				else
 					return GNOME_Evolution_Addressbook_OtherError;
 			}
 
-			db_error = db_create (&db, env, 0);
-			if (db_error != 0) {
- 				g_warning ("db_create failed with %s", db_strerror (db_error));
-				g_free (dirname);
-				g_free (filename);
-				return GNOME_Evolution_Addressbook_OtherError;
-			}
-
 			db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_CREATE | DB_THREAD, 0666);
 			if (db_error != 0) {
-				db->close (db, 0);
-				g_warning ("db->open (... DB_CREATE ...) failed with %s", db_strerror (db_error));
+				g_warning ("db->open (... DB_CREATE ...) failed with %d", db_error);
 			}
 			else {
 #ifdef CREATE_DEFAULT_VCARD
-				EContact *contact = NULL;
+				EContact *contact;
 
-				do_create(bf, XIMIAN_VCARD, &contact);
+				contact = do_create(bf, XIMIAN_VCARD);
 				/* XXX check errors here */
-				if (contact)
-					g_object_unref (contact);
+				g_object_unref (contact);
 #endif
 
 				writable = TRUE;
@@ -1300,7 +1170,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 		bf->priv->file_db = NULL;
 		g_free (dirname);
 		g_free (filename);
-		return db_error_to_status (db_error);
+		return GNOME_Evolution_Addressbook_OtherError;
 	}
 
 	if (!e_book_backend_file_maybe_upgrade_db (bf)) {
@@ -1425,22 +1295,6 @@ e_book_backend_file_set_mode (EBookBackend *backend,  GNOME_Evolution_Addressboo
 		e_book_backend_notify_connection_status (backend, TRUE);
 	}
 }
-
-static void 
-e_book_backend_file_sync (EBookBackend *backend)
-{
-	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
-	int db_error;
-	
-	g_return_if_fail (bf != NULL);
-	
-	if (bf->priv->file_db) {
-		db_error = bf->priv->file_db->sync (bf->priv->file_db, 0);
-		if (db_error != 0)
-			g_warning (G_STRLOC ": db->sync failed with %s", db_strerror (db_error));
-	}
-}
-
 static gboolean
 e_book_backend_file_construct (EBookBackendFile *backend)
 {
@@ -1479,43 +1333,31 @@ e_book_backend_file_dispose (GObject *object)
 
 	bf = E_BOOK_BACKEND_FILE (object);
 
-	if (bf->priv->file_db) {
-		bf->priv->file_db->close (bf->priv->file_db, 0);
-		bf->priv->file_db = NULL;
-	}
-	
-	g_static_mutex_lock(&global_env_lock);
-	global_env.ref_count--;
-	if (global_env.ref_count == 0) {
-		global_env.env->close (global_env.env, 0);
-		global_env.env = NULL;
-	}
-	g_static_mutex_unlock (&global_env_lock);
-	
-	if (bf->priv->summary) {
-		g_object_unref (bf->priv->summary);
-		bf->priv->summary = NULL;
+	if (bf->priv) {
+		if (bf->priv->file_db)
+			bf->priv->file_db->close (bf->priv->file_db, 0);
+
+		g_static_mutex_lock(&global_env_lock);
+		global_env.ref_count--;
+		if (global_env.ref_count == 0) {
+			global_env.env->close(global_env.env, 0);
+			global_env.env = NULL;
+		}
+		g_static_mutex_unlock(&global_env_lock);
+
+		if (bf->priv->summary)
+			g_object_unref(bf->priv->summary);
+		g_free (bf->priv->filename);
+		g_free (bf->priv->dirname);
+		g_free (bf->priv->summary_filename);
+
+		g_free (bf->priv);
+		bf->priv = NULL;
 	}
 
 	G_OBJECT_CLASS (e_book_backend_file_parent_class)->dispose (object);	
 }
 
-static void
-e_book_backend_file_finalize (GObject *object)
-{
-	EBookBackendFile *bf;
-
-	bf = E_BOOK_BACKEND_FILE (object);
-
-	g_free (bf->priv->filename);
-	g_free (bf->priv->dirname);
-	g_free (bf->priv->summary_filename);
-	
-	g_free (bf->priv);
-
-	G_OBJECT_CLASS (e_book_backend_file_parent_class)->finalize (object);
-}
-
 #ifdef G_OS_WIN32
 /* Avoid compiler warning by providing a function with exactly the
  * prototype that db_env_set_func_open() wants for the open method.
@@ -1535,29 +1377,6 @@ my_open (const char *name, int oflag, ...)
 
 	return g_open (name, oflag, mode);
 }
-
-int
-my_rename (const char *oldname, const char *newname)
-{
-	return g_rename (oldname, newname);
-}
-
-int
-my_exists (const char *name, int *isdirp)
-{
-	if (!g_file_test (name, G_FILE_TEST_EXISTS))
-		return ENOENT;
-	if (isdirp != NULL)
-		*isdirp = g_file_test (name, G_FILE_TEST_IS_DIR);
-	return 0;
-}
-
-int
-my_unlink (const char *name)
-{
-	return g_unlink (name);
-}
-
 #endif
 
 static void
@@ -1579,12 +1398,10 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	backend_class->stop_book_view          = e_book_backend_file_stop_book_view;
 	backend_class->cancel_operation        = e_book_backend_file_cancel_operation;
 	backend_class->set_mode                = e_book_backend_file_set_mode;
-	backend_class->sync                    = e_book_backend_file_sync;
 	sync_class->remove_sync                = e_book_backend_file_remove;
 	sync_class->create_contact_sync        = e_book_backend_file_create_contact;
 	sync_class->remove_contacts_sync       = e_book_backend_file_remove_contacts;
 	sync_class->modify_contact_sync        = e_book_backend_file_modify_contact;
-	sync_class->modify_contacts_sync        = e_book_backend_file_modify_contacts;
 	sync_class->get_contact_sync           = e_book_backend_file_get_contact;
 	sync_class->get_contact_list_sync      = e_book_backend_file_get_contact_list;
 	sync_class->get_changes_sync           = e_book_backend_file_get_changes;
@@ -1594,17 +1411,11 @@ e_book_backend_file_class_init (EBookBackendFileClass *klass)
 	
 
 	object_class->dispose = e_book_backend_file_dispose;
-	object_class->finalize = e_book_backend_file_finalize;
 
 #ifdef G_OS_WIN32
-	/* Use the gstdio wrappers to open, check, rename and unlink
-	 * files from libdb.
-	 */
+	/* Use the gstdio wrapper for open() to open files in libdb */
 	db_env_set_func_open (my_open);
 	db_env_set_func_close (close);
-	db_env_set_func_exists (my_exists);
-	db_env_set_func_rename (my_rename);
-	db_env_set_func_unlink (my_unlink);
 #endif
 }
 
