@@ -6,15 +6,15 @@
  * Author: Federico Mena-Quintero <federico@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
@@ -55,13 +55,13 @@ enum {
 	DEFAULT_OBJECT,
 	OBJECT,
 	OBJECT_LIST,
+	ATTACHMENT_LIST,
 	GET_TIMEZONE,
 	ADD_TIMEZONE,
 	SET_DEFAULT_TIMEZONE,
 	GET_CHANGES,
 	GET_FREE_BUSY,
 	QUERY,
-	CATEGORIES_CHANGED,
 	AUTH_REQUIRED,
 	BACKEND_ERROR,
 	LAST_SIGNAL
@@ -93,6 +93,9 @@ convert_status (const GNOME_Evolution_Calendar_CallStatus status)
 		return E_CALENDAR_STATUS_AUTHENTICATION_REQUIRED;
 	case GNOME_Evolution_Calendar_UnknownUser:
 		return E_CALENDAR_STATUS_UNKNOWN_USER;
+	case GNOME_Evolution_Calendar_InvalidServerVersion:
+		return E_CALENDAR_STATUS_INVALID_SERVER_VERSION;
+	
 	case GNOME_Evolution_Calendar_OtherError:
 	default:
 		return E_CALENDAR_STATUS_OTHER_ERROR;
@@ -418,6 +421,33 @@ impl_notifyObjectListRequested (PortableServer_Servant servant,
 	g_list_free (object_list);
 }
 
+static void 
+impl_notifyAttachmentListRequested (PortableServer_Servant servant,
+				const GNOME_Evolution_Calendar_CallStatus status,
+				const GNOME_Evolution_Calendar_stringlist *attachments,
+				CORBA_Environment *ev) 
+{
+	ECalListener *listener;
+	ECalListenerPrivate *priv;
+	GSList *a_list = NULL;
+	int i;
+	
+	listener = E_CAL_LISTENER (bonobo_object_from_servant (servant));
+	priv = listener->priv;
+
+	if (!priv->notify)
+		return;
+
+	for (i = 0; i < attachments->_length; i++) {
+		a_list = g_slist_append (a_list, g_strdup (attachments->_buffer[i]));
+	}
+
+	g_signal_emit (G_OBJECT (listener), signals[ATTACHMENT_LIST], 0, convert_status (status), a_list);
+
+	g_slist_foreach (a_list, (GFunc) g_free, NULL);
+	g_slist_free (a_list);
+}
+
 static void
 impl_notifyTimezoneRequested (PortableServer_Servant servant,
 			      const GNOME_Evolution_Calendar_CallStatus status,
@@ -662,24 +692,6 @@ impl_notifyErrorOccurred (PortableServer_Servant servant,
 	g_signal_emit (G_OBJECT (listener), signals[BACKEND_ERROR], 0, message);
 }
 
-/* ::notifyCategoriesChanged method */
-static void
-impl_notifyCategoriesChanged (PortableServer_Servant servant,
-			      const GNOME_Evolution_Calendar_StringSeq *categories,
-			      CORBA_Environment *ev)
-{
-	ECalListener *listener;
-	ECalListenerPrivate *priv;
-
-	listener = E_CAL_LISTENER (bonobo_object_from_servant (servant));
-	priv = listener->priv;
-
-	if (!priv->notify)
-		return;
-
-	g_signal_emit (G_OBJECT (listener), signals[CATEGORIES_CHANGED], 0, categories);
-}
-
 
 
 /* Object initialization function for the calendar listener */
@@ -744,6 +756,7 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 	klass->epv.notifyDefaultObjectRequested = impl_notifyDefaultObjectRequested;
 	klass->epv.notifyObjectRequested = impl_notifyObjectRequested;
 	klass->epv.notifyObjectListRequested = impl_notifyObjectListRequested;
+	klass->epv.notifyAttachmentListRequested = impl_notifyAttachmentListRequested;
 	klass->epv.notifyTimezoneRequested = impl_notifyTimezoneRequested;
 	klass->epv.notifyTimezoneAdded = impl_notifyTimezoneAdded;
 	klass->epv.notifyDefaultTimezoneSet = impl_notifyDefaultTimezoneSet;
@@ -752,7 +765,6 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 	klass->epv.notifyQuery = impl_notifyQuery;
 	klass->epv.notifyCalSetMode = impl_notifyCalSetMode;
 	klass->epv.notifyErrorOccurred = impl_notifyErrorOccurred;
-	klass->epv.notifyCategoriesChanged = impl_notifyCategoriesChanged;
 	klass->epv.notifyAuthRequired      = impl_notifyAuthRequired;
 	object_class->finalize = e_cal_listener_finalize;
 
@@ -883,6 +895,14 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 			      G_STRUCT_OFFSET (ECalListenerClass, object_list),
 			      NULL, NULL,
 			      e_cal_marshal_VOID__INT_POINTER,
+		      	      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_POINTER);
+	signals[ATTACHMENT_LIST] =
+		g_signal_new ("attachment_list",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (ECalListenerClass, attachment_list),
+			      NULL, NULL,
+			      e_cal_marshal_VOID__INT_POINTER,
 			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_POINTER);
 	signals[GET_TIMEZONE] =
 		g_signal_new ("get_timezone",
@@ -932,14 +952,6 @@ e_cal_listener_class_init (ECalListenerClass *klass)
 			      NULL, NULL,
 			      e_cal_marshal_VOID__INT_POINTER,
 			      G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_POINTER);
-	signals[CATEGORIES_CHANGED] =
-		g_signal_new ("categories_changed",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (ECalListenerClass, categories_changed),
-			      NULL, NULL,
-			      e_cal_marshal_VOID__POINTER,
-			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals [AUTH_REQUIRED] =
 		g_signal_new ("auth_required",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -963,6 +975,20 @@ BONOBO_TYPE_FUNC_FULL (ECalListener,
 		       BONOBO_TYPE_OBJECT,
 		       e_cal_listener);
 
+/**
+ * e_cal_listener_construct:
+ * @listener: A calendar listener.
+ * @cal_set_mode_fn: Function callback for notification that a
+ * calendar changed modes.
+ * @fn_data: Closure data pointer that will be passed to the notification
+ * functions.
+ *
+ * Constructs all internal information for a calendar listener. This function
+ * usually does not need to be called, unless creating a ECalListener-derived
+ * class.
+ *
+ * Return value: the calendar listener ready to be used.
+ */
 ECalListener *
 e_cal_listener_construct (ECalListener *listener,
 			  ECalListenerCalSetModeFn cal_set_mode_fn,
@@ -985,8 +1011,8 @@ e_cal_listener_construct (ECalListener *listener,
 /**
  * e_cal_listener_new:
  * @cal_set_mode_fn: Function callback for notification that a
- * calendar changed modes
- * @fn_data: losure data pointer that will be passed to the notification
+ * calendar changed modes.
+ * @fn_data: Closure data pointer that will be passed to the notification
  * functions.
  * 
  * Creates a new #ECalListener object.
