@@ -23,13 +23,10 @@
  * USA
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h> 
-#endif
 
 #include <stdlib.h>
 #include <sys/types.h>
-#include <dirent.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
