@@ -22,6 +22,7 @@
 #ifndef __E_UTIL_H__
 #define __E_UTIL_H__
 
+#include <pthread.h>
 #include <sys/types.h>
 #include <glib/gmacros.h>
 #include <glib/gtypes.h>
@@ -37,10 +38,20 @@ gchar       *e_util_strstrcase (const gchar *haystack, const gchar *needle);
 gchar       *e_util_unicode_get_utf8 (const gchar *text, gunichar *out);
 const gchar *e_util_utf8_strstrcase (const gchar *haystack, const gchar *needle);
 const gchar *e_util_utf8_strstrcasedecomp (const gchar *haystack, const gchar *needle);
+int	     e_util_utf8_strcasecmp (const gchar *s1, const gchar *s2);
+guint64      e_util_pthread_id (pthread_t t);
+void         e_filename_make_safe (gchar *string);
 
 size_t e_utf8_strftime(char *s, size_t max, const char *fmt, const struct tm *tm);
 size_t e_strftime(char *s, size_t max, const char *fmt, const struct tm *tm);
 
+#ifdef G_OS_WIN32
+const char  *e_util_get_prefix (void) G_GNUC_CONST;
+const char  *e_util_get_cp_prefix (void) G_GNUC_CONST;
+const char  *e_util_get_localedir (void) G_GNUC_CONST;
+char        *e_util_replace_prefix (const char *runtime_prefix, const char *configure_time_path);
+#endif
+
 G_END_DECLS
 
 #endif
