@@ -5,7 +5,7 @@
  * Copyright (C) 2004 Novell, Inc.
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,7 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -28,6 +28,7 @@
 #include <gtk/gtkcelllayout.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtkradiomenuitem.h>
 #include <gtk/gtkseparatormenuitem.h>
 #include <glib/gi18n-lib.h>
 
@@ -35,12 +36,13 @@
 #include <libebook/e-contact.h>
 #include <libebook/e-destination.h>
 #include <libedataserverui/e-book-auth-util.h>
+#include <libedataserver/e-sexp.h>
 
 #include "e-name-selector-entry.h"
 
 #define ENS_DEBUG(x)
 
-#define COMPLETION_CUE_MIN_LEN 2
+#define COMPLETION_CUE_MIN_LEN 3
 
 G_DEFINE_TYPE (ENameSelectorEntry, e_name_selector_entry, GTK_TYPE_ENTRY);
 
@@ -86,8 +88,6 @@ e_name_selector_entry_realize (GtkWidget *widget)
 static void
 e_name_selector_entry_dispose (GObject *object)
 {
-	ENameSelectorEntry *name_selector_entry = E_NAME_SELECTOR_ENTRY (object);
-
 	if (G_OBJECT_CLASS (e_name_selector_entry_parent_class)->dispose)
 		G_OBJECT_CLASS (e_name_selector_entry_parent_class)->dispose (object);
 }
@@ -96,8 +96,6 @@ e_name_selector_entry_dispose (GObject *object)
 static void
 e_name_selector_entry_finalize (GObject *object)
 {
-	ENameSelectorEntry *name_selector_entry = E_NAME_SELECTOR_ENTRY (object);
-
 	if (G_OBJECT_CLASS (e_name_selector_entry_parent_class)->finalize)
 		G_OBJECT_CLASS (e_name_selector_entry_parent_class)->finalize (object);
 }
@@ -300,7 +298,6 @@ get_address_at_position (const gchar *string, gint pos)
 	gint         end_pos;
 	const gchar *start_p;
 	const gchar *end_p;
-	gchar       *address;
 
 	if (!get_range_at_position (string, pos, &start_pos, &end_pos))
 		return NULL;
@@ -352,7 +349,6 @@ build_destination_at_position (const gchar *string, gint pos)
 {
 	EDestination *destination;
 	gchar        *address;
-	gint          i;
 
 	address = get_address_at_position (string, pos);
 	if (!address)
@@ -373,7 +369,6 @@ name_style_query (const gchar *field, const gchar *value)
 	GString *out = g_string_new ("");
 	gchar  **strv;
 	gchar   *query;
-	gint     i;
 
 	spaced_str = sanitize_string (value);
 	g_strstrip (spaced_str);
@@ -474,7 +469,6 @@ get_entry_substring (ENameSelectorEntry *name_selector_entry, gint range_start,
 {
 	const gchar *entry_text;
 	gchar       *p0, *p1;
-	gchar       *substr;
 
 	entry_text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 
@@ -511,7 +505,6 @@ build_textrep_for_contact (EContact *contact, EContactField cue_field)
 	gchar *name  = NULL;
 	gchar *email = NULL;
 	gchar *textrep;
-	gchar *p0;
 
 	switch (cue_field) {
 		case E_CONTACT_FULL_NAME:
@@ -617,7 +610,7 @@ find_existing_completion (ENameSelectorEntry *name_selector_entry, const gchar *
 	GtkTreeIter    iter;
 	EContact      *best_contact    = NULL;
 	gint           best_field_rank = G_MAXINT;
-	EContactField  best_field;
+	EContactField  best_field = 0;
 	gint           cue_len;
 
 	g_assert (cue_str);
@@ -851,7 +844,6 @@ insert_destination_at_position (ENameSelectorEntry *name_selector_entry, gint po
 	EDestination *destination;
 	const gchar  *text;
 	gint          index;
-	gint          range_start, range_end;
 
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 	index = get_index_at_position (text, pos);
@@ -874,8 +866,6 @@ modify_destination_at_position (ENameSelectorEntry *name_selector_entry, gint po
 	EDestination *destination;
 	const gchar  *text;
 	gchar        *raw_address;
-	gint          index;
-	gint          range_start, range_end;
 	gboolean      rebuild_attributes = FALSE;
 
 	destination = find_destination_at_position (name_selector_entry, pos);
@@ -906,14 +896,14 @@ remove_destination_at_position (ENameSelectorEntry *name_selector_entry, gint po
 	EDestination *destination;
 
 	destination = find_destination_at_position (name_selector_entry, pos);
-	g_assert (destination);
-
-	g_signal_handlers_block_by_func (name_selector_entry->destination_store,
+	if (destination) {
+		g_signal_handlers_block_by_func (name_selector_entry->destination_store,
 					 destination_row_deleted, name_selector_entry);
-	e_destination_store_remove_destination (name_selector_entry->destination_store,
+		e_destination_store_remove_destination (name_selector_entry->destination_store,
 						destination);
-	g_signal_handlers_unblock_by_func (name_selector_entry->destination_store,
+		g_signal_handlers_unblock_by_func (name_selector_entry->destination_store,
 					   destination_row_deleted, name_selector_entry);
+	}
 }
 
 static void
@@ -967,14 +957,14 @@ remove_destination_by_index (ENameSelectorEntry *name_selector_entry, gint index
 	EDestination *destination;
 
 	destination = find_destination_by_index (name_selector_entry, index);
-	g_assert (destination);
-
-	g_signal_handlers_block_by_func (name_selector_entry->destination_store,
+	if (destination) {
+		g_signal_handlers_block_by_func (name_selector_entry->destination_store,
 					 destination_row_deleted, name_selector_entry);
-	e_destination_store_remove_destination (name_selector_entry->destination_store,
+		e_destination_store_remove_destination (name_selector_entry->destination_store,
 						destination);
-	g_signal_handlers_unblock_by_func (name_selector_entry->destination_store,
+		g_signal_handlers_unblock_by_func (name_selector_entry->destination_store,
 					   destination_row_deleted, name_selector_entry);
+	}
 }
 
 /* Returns the number of characters inserted */
@@ -1001,7 +991,6 @@ insert_unichar (ENameSelectorEntry *name_selector_entry, gint *pos, gunichar c)
 	 * - At start of string. */
 
 	if (c == ',' && !is_quoted_at (text, *pos)) {
-		const gchar *p0;
 		gint         start_pos;
 		gint         end_pos;
 		gboolean     at_start = FALSE;
@@ -1123,17 +1112,26 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 {
 	const gchar *text;
 	gint         index_start, index_end;
-	gunichar     str_context [2];
-	gchar        buf [7];
+	gint	     selection_start, selection_end;	
+	gunichar     str_context [2], str_b_context [2];;
 	gint         len;
 	gint         i;
+	gboolean     already_selected = FALSE;
 
-	if (start_pos == end_pos)
+	if (start_pos == end_pos) 
 		return;
 
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
 	len = g_utf8_strlen (text, -1);
+
+	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (name_selector_entry), 
+					       &selection_start, 
+					       &selection_end)) 
+		if (g_utf8_get_char (g_utf8_offset_to_pointer (text, selection_end - 1)) == ',') 
+			already_selected = TRUE;
+	
 	get_utf8_string_context (text, start_pos, str_context, 2);
+	get_utf8_string_context (text, end_pos-1, str_b_context, 2);
 
 	g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
 
@@ -1146,14 +1144,40 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 		}
 	}
 
-	if (str_context [0] == ',' && str_context [1] == ' ') {
-		/* If we're deleting the trailing space in ", ", delete the whole ", " sequence. */
-		start_pos--;
-	}
-
 	index_start = get_index_at_position (text, start_pos);
 	index_end   = get_index_at_position (text, end_pos);
+	
+	g_signal_stop_emission_by_name (name_selector_entry, "delete_text");
+	
+	/* If the user is trying to delete a ','-character, we assume the user 
+	 * wants to remove the entire destination.
+	 */
 
+	if (((str_b_context [0] == ',' && str_b_context [1] == ' ') || str_b_context [1] == ',') && !already_selected) {
+
+		EDestination *dest = find_destination_at_position (name_selector_entry, end_pos-1);
+	  	const char *email = e_destination_get_email (dest);
+	  	if (email && (strcmp (email, "")!=0)) {
+	  
+			/* Therefore, in case it's a real destination, we select it. 
+		 	 * Deleting this selection afterwards will leave the destination
+		 	 * empty. */
+	 
+			gint t = (str_b_context [1]==',')?end_pos:end_pos-1, b=t;
+			do {
+				t--;
+			} while (t >= 1 && text[t-1] != ',');
+	
+			gtk_editable_select_region (GTK_EDITABLE(name_selector_entry), t, b);
+
+			/* Since this is a special-case, we don't want the rest of this method
+		 	 * to happen. However, we do need to reenable the signal which we
+		 	 * disabled above! */
+		
+			goto end_of_user_delete_text;
+	  	}
+	}
+	
 	/* If the deletion touches more than one destination, the first one is changed
 	 * and the rest are removed. If the last destination wasn't completely deleted,
 	 * it becomes part of the first one, since the separator between them was
@@ -1162,15 +1186,17 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 	 * Here, we let the model know about removals. */
 	for (i = index_end; i > index_start; i--)
 		remove_destination_by_index (name_selector_entry, i);
-
+	
 	/* Do the actual deletion */
+	
+	
 	gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry),
 				  start_pos, end_pos);
-	g_signal_stop_emission_by_name (name_selector_entry, "delete_text");
+	
 
 	/* Let model know about changes */
 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));
-	if (!*text) {
+	if (!*text || strlen(text) <= 0) {
 		/* If the entry was completely cleared, remove the initial destination too */
 		remove_destination_by_index (name_selector_entry, 0);
 		generate_attribute_list (name_selector_entry);
@@ -1187,7 +1213,8 @@ user_delete_text (ENameSelectorEntry *name_selector_entry, gint start_pos, gint
 		g_source_remove (name_selector_entry->type_ahead_complete_cb_id);
 		name_selector_entry->type_ahead_complete_cb_id = 0;
 	}
-
+	
+end_of_user_delete_text:
 	g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
 }
 
@@ -1195,13 +1222,9 @@ static gboolean
 completion_match_selected (ENameSelectorEntry *name_selector_entry, GtkTreeModel *model,
 			   GtkTreeIter *iter)
 {
-	EContactStore *contact_store;
 	EContact      *contact;
 	EDestination  *destination;
-	EContactField  matched_field = E_CONTACT_EMAIL_1;
 	gint           cursor_pos;
-	const gchar   *text;
-	gchar         *raw_address;
 	GtkTreeIter    generator_iter;
 	GtkTreeIter    contact_iter;
 	gint           email_n;
@@ -1223,6 +1246,7 @@ completion_match_selected (ENameSelectorEntry *name_selector_entry, GtkTreeModel
 	destination = find_destination_at_position (name_selector_entry, cursor_pos);
 	e_destination_set_contact (destination, contact, email_n);
 	sync_destination_at_position (name_selector_entry, cursor_pos, &cursor_pos);
+	gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), ",", -1, &cursor_pos);
 
 	/* Place cursor at end of address */
 
@@ -1339,6 +1363,16 @@ generate_contact_rows (EContactStore *contact_store, GtkTreeIter *iter,
 	return n_rows;
 }
 
+static void
+ensure_type_ahead_complete_on_idle (ENameSelectorEntry *name_selector_entry)
+{
+	if (!name_selector_entry->type_ahead_complete_cb_id) {
+		name_selector_entry->type_ahead_complete_cb_id =
+			g_idle_add ((GSourceFunc) type_ahead_complete_on_idle_cb,
+				    name_selector_entry);
+	}
+}
+
 static void
 setup_contact_store (ENameSelectorEntry *name_selector_entry)
 {
@@ -1359,6 +1393,10 @@ setup_contact_store (ENameSelectorEntry *name_selector_entry)
 
 		gtk_entry_completion_set_model (name_selector_entry->entry_completion,
 						GTK_TREE_MODEL (name_selector_entry->email_generator));
+
+		/* Set up callback for incoming matches */
+		g_signal_connect_swapped (name_selector_entry->contact_store, "row-inserted",
+					  G_CALLBACK (ensure_type_ahead_complete_on_idle), name_selector_entry);
 	} else {
 		/* Remove the store from the entry completion */
 
@@ -1569,7 +1607,6 @@ static void
 setup_destination_store (ENameSelectorEntry *name_selector_entry)
 {
 	GtkTreeIter  iter;
-	GtkTreePath *path;
 
 	g_signal_connect_swapped (name_selector_entry->destination_store, "row-changed",
 				  G_CALLBACK (destination_row_changed), name_selector_entry);
@@ -1626,6 +1663,8 @@ prepare_popup_destination (ENameSelectorEntry *name_selector_entry, GdkEventButt
 
 	index = gtk_entry_layout_index_to_text_index (GTK_ENTRY (name_selector_entry), index);
 	destination = find_destination_at_position (name_selector_entry, index);
+	/* FIXME: Add this to a private variable, in ENameSelectorEntry Class*/
+	g_object_set_data ((GObject *)name_selector_entry, "index", GINT_TO_POINTER(index));
 
 	if (!destination || !e_destination_get_contact (destination))
 		return FALSE;
@@ -1662,6 +1701,33 @@ editor_closed_cb (GtkObject *editor, gpointer data)
 	g_object_unref (editor);
 }
 
+static void
+popup_activate_inline_expand (ENameSelectorEntry *name_selector_entry, GtkWidget *menu_item)
+{
+	const char *email_list, *text;
+	EDestination *destination = name_selector_entry->popup_destination;
+	int position, start, end;
+
+ 	position = GPOINTER_TO_INT(g_object_get_data ((GObject *)name_selector_entry, "index"));
+	
+	email_list = e_destination_get_address(destination);
+ 	text = gtk_entry_get_text (GTK_ENTRY (name_selector_entry));	
+ 	get_range_at_position (text, position, &start, &end);
+
+	g_signal_handlers_block_by_func (name_selector_entry, user_delete_text, name_selector_entry);
+
+	gtk_editable_delete_text (GTK_EDITABLE (name_selector_entry), start, end);
+
+	text = sanitize_string (e_destination_get_textrep (destination, FALSE));
+	gtk_editable_insert_text (GTK_EDITABLE (name_selector_entry), email_list, -1, &start);
+
+	g_signal_handlers_unblock_by_func (name_selector_entry, user_delete_text, name_selector_entry);
+
+	clear_completion_model (name_selector_entry);
+	generate_attribute_list (name_selector_entry);
+
+}
+
 static void
 popup_activate_contact (ENameSelectorEntry *name_selector_entry, GtkWidget *menu_item)
 {
@@ -1670,7 +1736,6 @@ popup_activate_contact (ENameSelectorEntry *name_selector_entry, GtkWidget *menu
 	EDestination *destination;
 	EContact     *contact;
 	gchar        *contact_uid;
-	GtkTreeIter   iter;
 
 	destination = name_selector_entry->popup_destination;
 	if (!destination)
@@ -1683,9 +1748,9 @@ popup_activate_contact (ENameSelectorEntry *name_selector_entry, GtkWidget *menu
 	contact_uid = e_contact_get (contact, E_CONTACT_UID);
 	if (!contact_uid)
 		return;
-
 	if (name_selector_entry->contact_store) {
 		books = e_contact_store_get_books (name_selector_entry->contact_store);
+		/*FIXME: read URI from contact and get the book ?*/
 		book = find_book_by_contact (books, contact_uid);
 		g_list_free (books);
 		g_free (contact_uid);
@@ -1736,15 +1801,57 @@ popup_activate_email (ENameSelectorEntry *name_selector_entry, GtkWidget *menu_i
 	e_destination_set_contact (destination, contact, email_num);
 }
 
+static void
+popup_activate_list (EDestination *destination, GtkWidget *item)
+{
+	gboolean status = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item));
+	
+	e_destination_set_ignored (destination, !status);	
+}
+
+static void
+destination_set_list (GtkWidget *item, EDestination *destination)
+{
+	EContact *contact;
+	gboolean status = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item));
+
+	contact = e_destination_get_contact (destination);
+	if (!contact)
+		return;
+	
+	e_destination_set_ignored (destination, !status);
+}
+
+static void
+destination_set_email (GtkWidget *item, EDestination *destination)
+{
+	int email_num;
+	EContact *contact;
+
+ 	if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (item)))
+		return;
+	contact = e_destination_get_contact (destination);
+	if (!contact)
+		return;
+
+	email_num = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "order"));
+	e_destination_set_contact (destination, contact, email_num);
+}
+
 static void
 populate_popup (ENameSelectorEntry *name_selector_entry, GtkMenu *menu)
 {
 	EDestination *destination;
 	EContact     *contact;
 	GtkWidget    *menu_item;
-	GList        *email_list;
+	GList        *email_list=NULL;
 	GList        *l;
 	gint          i;
+	char 	     *edit_label;
+	int 	      email_num, len;
+	GSList 	     *group = NULL;
+	gboolean      is_list;
+	gboolean      show_menu = FALSE;
 
 	destination = name_selector_entry->popup_destination;
 	if (!destination)
@@ -1761,29 +1868,91 @@ populate_popup (ENameSelectorEntry *name_selector_entry, GtkMenu *menu)
 	menu_item = gtk_separator_menu_item_new ();
 	gtk_widget_show (menu_item);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+	email_num = e_destination_get_email_num (destination);
 
 	/* Addresses */
+	is_list = e_contact_get (contact, E_CONTACT_IS_LIST) ? TRUE : FALSE;
+	if (is_list) {
+		const GList *dests = e_destination_list_get_dests (destination);
+		GList *iter;
+		int len = g_list_length ((GList *)dests);
 
-	email_list = e_contact_get (contact, E_CONTACT_EMAIL);
+		for (iter = (GList *)dests; iter; iter = iter->next) {
+			EDestination *dest = (EDestination *) iter->data;
+			const char *email = e_destination_get_email (dest);
+			
+			if (!email || *email == '\0')
+				continue;	
 
-	for (l = email_list, i = 0; l; l = g_list_next (l), i++) {
-		gchar *email = l->data;
+			if (len > 1) {
+				menu_item = gtk_check_menu_item_new_with_label (email);
+				g_signal_connect (menu_item, "toggled", G_CALLBACK (destination_set_list), dest);				
+			} else {
+				menu_item = gtk_menu_item_new_with_label (email);
+			}
+			
+			gtk_widget_show (menu_item);
+			gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+			show_menu = TRUE;
 
-		if (!email || *email == '\0')
-			continue;
+			if ( len > 1 ) {
+				gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), !e_destination_is_ignored(dest));
+				g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (popup_activate_list),
+							  dest);	
+			}
+		}
+		
+	} else {
+		email_list = e_contact_get (contact, E_CONTACT_EMAIL);
+		len = g_list_length (email_list);
 
-		menu_item = gtk_menu_item_new_with_label (email);
-		gtk_widget_show (menu_item);
-		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+		for (l = email_list, i = 0; l; l = g_list_next (l), i++) {
+			gchar *email = l->data;
 
-		g_object_set_data (G_OBJECT (menu_item), "order", GINT_TO_POINTER (i));
-		g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (popup_activate_email),
-					  name_selector_entry);
+			if (!email || *email == '\0')
+				continue;
+		
+			if (len > 1) {
+				menu_item = gtk_radio_menu_item_new_with_label (group, email);
+				group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menu_item));
+				g_signal_connect (menu_item, "toggled", G_CALLBACK (destination_set_email), destination);
+			} else {
+				menu_item = gtk_menu_item_new_with_label (email);			
+			}
+			
+			gtk_widget_show (menu_item);
+			gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+			show_menu = TRUE;
+			g_object_set_data (G_OBJECT (menu_item), "order", GINT_TO_POINTER (i));			
+
+			if ( i == email_num && len > 1 ) {
+				gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), TRUE);
+				g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (popup_activate_email),
+							  name_selector_entry);
+			}
+		}
 	}
 
 	/* Separator */
 
-	if (email_list) {
+	if (show_menu) {
+		menu_item = gtk_separator_menu_item_new ();
+		gtk_widget_show (menu_item);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+	}
+
+	/* Expand a list inline */
+	if (is_list) {
+		/* To Translators: This would be similiar to "Expand MyList Inline" where MyList is a Contact List*/
+		edit_label = g_strdup_printf (_("E_xpand %s Inline"), (char *)e_contact_get_const (contact, E_CONTACT_FILE_AS)); 
+		menu_item = gtk_menu_item_new_with_mnemonic (edit_label);
+		g_free (edit_label);
+		gtk_widget_show (menu_item);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+		g_signal_connect_swapped (menu_item, "activate", G_CALLBACK (popup_activate_inline_expand),
+					  name_selector_entry);
+
+		/* Separator */
 		menu_item = gtk_separator_menu_item_new ();
 		gtk_widget_show (menu_item);
 		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
@@ -1791,7 +1960,9 @@ populate_popup (ENameSelectorEntry *name_selector_entry, GtkMenu *menu)
 
 	/* Edit Contact item */
 
-	menu_item = gtk_menu_item_new_with_label (e_contact_get_const (contact, E_CONTACT_FILE_AS));
+	edit_label = g_strdup_printf (_("_Edit %s"), (char *)e_contact_get_const (contact, E_CONTACT_FILE_AS)); 
+	menu_item = gtk_menu_item_new_with_mnemonic (edit_label);
+	g_free (edit_label);
 	gtk_widget_show (menu_item);
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
 
@@ -1860,12 +2031,27 @@ e_name_selector_entry_init (ENameSelectorEntry *name_selector_entry)
   setup_destination_store (name_selector_entry);
 }
 
+/**
+ * e_name_selector_entry_new:
+ *
+ * Creates a new #ENameSelectorEntry.
+ *
+ * Return value: A new #ENameSelectorEntry.
+ **/
 ENameSelectorEntry *
 e_name_selector_entry_new (void)
 {
 	  return g_object_new (e_name_selector_entry_get_type (), NULL);
 }
 
+/**
+ * e_name_selector_entry_peek_contact_store:
+ * @name_selector_entry: an #ENameSelectorEntry
+ *
+ * Gets the #EContactStore being used by @name_selector_entry.
+ *
+ * Return value: An #EContactStore.
+ **/
 EContactStore *
 e_name_selector_entry_peek_contact_store (ENameSelectorEntry *name_selector_entry)
 {
@@ -1874,6 +2060,13 @@ e_name_selector_entry_peek_contact_store (ENameSelectorEntry *name_selector_entr
 	return name_selector_entry->contact_store;
 }
 
+/**
+ * e_name_selector_entry_set_contact_store:
+ * @name_selector_entry: an #ENameSelectorEntry
+ * @contact_store: an #EContactStore to use
+ *
+ * Sets the #EContactStore being used by @name_selector_entry to @contact_store.
+ **/
 void
 e_name_selector_entry_set_contact_store (ENameSelectorEntry *name_selector_entry,
 					 EContactStore *contact_store)
@@ -1893,6 +2086,14 @@ e_name_selector_entry_set_contact_store (ENameSelectorEntry *name_selector_entry
 	setup_contact_store (name_selector_entry);
 }
 
+/**
+ * e_name_selector_entry_peek_destination_store:
+ * @name_selector_entry: an #ENameSelectorEntry
+ * 
+ * Gets the #EDestinationStore being used to store @name_selector_entry's destinations.
+ *
+ * Return value: An #EDestinationStore.
+ **/
 EDestinationStore *
 e_name_selector_entry_peek_destination_store (ENameSelectorEntry *name_selector_entry)
 {
@@ -1901,6 +2102,14 @@ e_name_selector_entry_peek_destination_store (ENameSelectorEntry *name_selector_
 	return name_selector_entry->destination_store;
 }
 
+/**
+ * e_name_selector_entry_set_destination_store:
+ * @name_selector_entry: an #ENameSelectorEntry
+ * @destination_store: an #EDestinationStore to use
+ *
+ * Sets @destination_store as the #EDestinationStore to be used to store
+ * destinations for @name_selector_entry.
+ **/
 void
 e_name_selector_entry_set_destination_store  (ENameSelectorEntry *name_selector_entry,
 					      EDestinationStore *destination_store)
@@ -1917,12 +2126,22 @@ e_name_selector_entry_set_destination_store  (ENameSelectorEntry *name_selector_
 	setup_destination_store (name_selector_entry);
 }
 
+/**
+ * e_name_selector_entry_set_contact_editor_func:
+ *
+ * DO NOT USE.
+ **/
 void
 e_name_selector_entry_set_contact_editor_func (ENameSelectorEntry *name_selector_entry, gpointer func)
 {
 	name_selector_entry->contact_editor_func = func;
 }
 
+/**
+ * e_name_selector_entry_set_contact_list_editor_func:
+ *
+ * DO NOT USE.
+ **/
 void
 e_name_selector_entry_set_contact_list_editor_func (ENameSelectorEntry *name_selector_entry, gpointer func)
 {
