@@ -3,7 +3,7 @@
 /* Copyright (C) 2003, 2004 Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -37,10 +37,21 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 
+#ifndef G_OS_WIN32
 #include <netinet/in.h>
 #include <arpa/nameser.h>
 #include <resolv.h>
+#else
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#include <windns.h>
+#ifndef DNS_TYPE_SRV
+#define DNS_TYPE_SRV 33
+#endif
+#endif
 
 #include "e2k-autoconfig.h"
 #include "e2k-encoding-utils.h"
@@ -52,18 +63,19 @@
 #include "e2k-xml-utils.h"
 #include "xntlm.h"
 
-//#include <e-util/e-account.h>
-//#include <e-util/e-account-list.h>
-#include <libedataserver/e-account.h>
-#include <libedataserver/e-account-list.h>
+#include <libedataserver/e-util.h>
 #include <libedataserverui/e-passwords.h>
-//#include <e-util/e-dialog-utils.h>
 #include <gconf/gconf-client.h>
 #include <libxml/tree.h>
 #include <libxml/HTMLparser.h>
 
 #include <gtk/gtk.h>
 
+#ifdef G_OS_WIN32
+#undef CONNECTOR_PREFIX
+#define CONNECTOR_PREFIX e_util_get_prefix ()
+#endif
+
 static char *find_olson_timezone (const char *windows_timezone);
 static void set_account_uri_string (E2kAutoconfig *ac);
 
@@ -288,7 +300,6 @@ e2k_autoconfig_set_password (E2kAutoconfig *ac, const char *password)
 	ac->password = g_strdup (password);
 }
 
-
 static void
 get_ctx_auth_handler (SoupMessage *msg, gpointer user_data)
 {
@@ -768,6 +779,7 @@ e2k_autoconfig_check_exchange (E2kAutoconfig *ac, E2kOperation *op)
 static void
 find_global_catalog (E2kAutoconfig *ac)
 {
+#ifndef G_OS_WIN32
 	int count, len;
 	unsigned char answer[1024], namebuf[1024], *end, *p;
 	guint16 type, qclass, rdlength, priority, weight, port;
@@ -820,6 +832,35 @@ find_global_catalog (E2kAutoconfig *ac)
 	}
 
 	return;
+#else
+	gchar *name, *casefolded_name;
+	PDNS_RECORD dnsrecp, rover;
+
+	name = g_strconcat ("_gc._tcp.", ac->w2k_domain, NULL);
+	casefolded_name = g_utf8_strdown (name, -1);
+	g_free (name);
+
+	if (DnsQuery_UTF8 (casefolded_name, DNS_TYPE_SRV, DNS_QUERY_STANDARD,
+			   NULL, &dnsrecp, NULL) != ERROR_SUCCESS) {
+		g_free (casefolded_name);
+		return;
+	}
+
+	for (rover = dnsrecp; rover != NULL; rover = rover->pNext) {
+		if (rover->wType != DNS_TYPE_SRV ||
+		    strcmp (rover->pName, casefolded_name) != 0)
+			continue;
+		ac->gc_server = g_strdup (rover->Data.SRV.pNameTarget);
+		ac->gc_server_autodetected = TRUE;
+		g_free (casefolded_name);
+		DnsRecordListFree (dnsrecp, DnsFreeRecordList);
+		return;
+	}
+
+	g_free (casefolded_name);
+	DnsRecordListFree (dnsrecp, DnsFreeRecordList);
+	return;
+#endif
 }
 
 /**
@@ -1282,13 +1323,20 @@ find_olson_timezone (const char *windows_timezone)
 		return g_strdup (zonemap[i].olson_name);
 
 	/* Find our language/country (hopefully). */
+#ifndef G_OS_WIN32
 	locale = getenv ("LANG");
+#else
+	locale = g_win32_getlocale ();
+#endif
 	if (locale) {
 		strncpy (lang, locale, 2);
 		locale = strchr (locale, '_');
 		if (locale++)
 			strncpy (country, locale, 2);
 	}
+#ifdef G_OS_WIN32
+	g_free ((char *) locale);
+#endif
 
 	/* Look for an entry where either the country or the
 	 * language matches.
@@ -1319,14 +1367,22 @@ read_config (void)
 	struct stat st;
 	char *p, *name, *value;
 	char *config_data;
-	int fd;
+	int fd = -1;
 
 	config_options = g_hash_table_new (e2k_ascii_strcase_hash,
 					    e2k_ascii_strcase_equal);
 
-	fd = open ("/etc/ximian/connector.conf", O_RDONLY);
-	if (fd == -1)
-		fd = open (CONNECTOR_PREFIX "/etc/connector.conf", O_RDONLY);
+#ifndef G_OS_WIN32
+	fd = g_open ("/etc/ximian/connector.conf", O_RDONLY, 0);
+#endif
+	if (fd == -1) {
+		gchar *filename = g_build_filename (CONNECTOR_PREFIX,
+						    "etc/connector.conf",
+						    NULL);
+		
+		fd = g_open (filename, O_RDONLY, 0);
+		g_free (filename);
+	}
 	if (fd == -1)
 		return;
 	if (fstat (fd, &st) == -1) {
@@ -1448,9 +1504,9 @@ validate (const char *owa_url, char *user, char *password, ExchangeParams *excha
 		exchange_params->mailbox  = g_strdup (mailbox);
 		exchange_params->owa_path = g_strdup_printf ("%s%s", "/", path);
 		g_free (path);
-		exchange_params->host = ac->pf_server;
+		exchange_params->host = g_strdup (ac->pf_server);
 		if (ac->gc_server) 
-			exchange_params->ad_server = ac->gc_server;
+			exchange_params->ad_server = g_strdup (ac->gc_server);
 		exchange_params->is_ntlm = ac->saw_ntlm;
 
 		valid = TRUE;
@@ -1555,7 +1611,7 @@ validate (const char *owa_url, char *user, char *password, ExchangeParams *excha
 		}
 	}
 
-
+	e2k_autoconfig_free (ac);
 	return valid;
 }
 
@@ -1567,7 +1623,8 @@ e2k_validate_user (const char *owa_url, char *user,
 	gboolean valid = FALSE, remember=FALSE;
 	char *key, *password, *prompt;
 
-	key = g_strdup_printf ("%s//%s@%s", "exchange:", user, owa_url); /* FIXME */
+	key = g_strdup_printf ("%s//%s@%s/", "exchange:", user, owa_url); /* FIXME */
+		
 	password = e_passwords_get_password ("Exchange", key);
 	if (!password) {
 		prompt = g_strdup_printf (_("Enter password for %s"), user);
@@ -1575,30 +1632,36 @@ e2k_validate_user (const char *owa_url, char *user,
 					"Exchange", key, prompt,
 					E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET,
 					&remember, NULL);
-		if (password) {
-			valid = validate (owa_url, user, password, exchange_params, result);
-			if (valid) {
-				/* generate the proper key once the host name 
-				 * is read and remember password temporarily, 
-				 * so that at the end of * account creation, 
-				 * user will not be prompted, for password will
-				 * not be asked again. 
-				 */
-
-				*remember_password = remember;
-				g_free (key);
-				key = g_strdup_printf ("%s//%s@%s", 
-						       "exchange:", user, exchange_params->host);
-				e_passwords_add_password (key, password);
-				e_passwords_remember_password ("Exchange", key);
-			}
-		}
 		g_free (prompt);
+		if (!password) {
+			g_free (key);
+			return valid;
+		}
 	}
-	if (password && !valid) {
+
+	valid = validate (owa_url, user, password, exchange_params, result);
+	if (valid) {
+		/* generate the proper key once the host name 
+		 * is read and remember password temporarily, 
+		 * so that at the end of * account creation, 
+		 * user will not be prompted, for password will
+		 * not be asked again. 
+		 */
+		*remember_password = remember;
+		g_free (key);
+		if (exchange_params->is_ntlm)
+			key = g_strdup_printf ("exchange://%s;auth=NTLM@%s/", 
+						       user, exchange_params->host);
+		else
+			key = g_strdup_printf ("exchange://%s@%s/", user, exchange_params->host);
+		e_passwords_add_password (key, password);
+		e_passwords_remember_password ("Exchange", key);
+	}
+	else {
+		/* if validation failed*/
 		e_passwords_forget_password ("Exchange", key);
 	}
-	g_free (key);
 
+	g_free (key);
 	return valid;
 }
