@@ -3,7 +3,7 @@
 /* Copyright (C) 2002-2004 Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -21,30 +21,37 @@
 #include <config.h>
 #endif
 
-#include "e-folder-exchange.h"
-#include "exchange-account.h"
-#include "exchange-hierarchy.h"
-#include "e2k-uri.h"
-#include "e2k-path.h"
-
-#include <libedataserver/e-util.h>
-#include <libedataserver/e-xml-hash-utils.h>
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
-#include <libxml/tree.h>
-#include <libedataserver/e-source-list.h>
-
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include <libxml/parser.h>
+#include <libxml/tree.h>
+#include <libxml/xmlmemory.h>
+
+#include <glib.h>
+#include <glib/gstdio.h>
+
+#include "libedataserver/e-source-list.h"
+#include "libedataserver/e-util.h"
+#include "libedataserver/e-xml-utils.h"
+
+#include "e-folder-exchange.h"
+#include "e2k-path.h"
+#include "e2k-uri.h"
+#include "exchange-account.h"
+#include "exchange-esource.h"
+#include "exchange-hierarchy.h"
+
+
 struct _EFolderExchangePrivate {
 	ExchangeHierarchy *hier;
 	char *internal_uri, *permanent_uri;
 	char *outlook_class, *storage_dir;
-	const char *path;
+	char *path;
 	long long int folder_size;
 	gboolean has_subfolders;
 };
@@ -97,6 +104,7 @@ finalize (GObject *object)
 	g_free (folder->priv->permanent_uri);
 	g_free (folder->priv->outlook_class);
 	g_free (folder->priv->storage_dir);
+	g_free (folder->priv->path);
 	g_free (folder->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -104,48 +112,26 @@ finalize (GObject *object)
 
 E2K_MAKE_TYPE (e_folder_exchange, EFolderExchange, class_init, init, PARENT_TYPE)
 
-/**
- * Added from gal/util/e-util.c
- * e_mkdir_hier:
- * @path: a directory path
- * @mode: a mode, as for mkdir(2)
- *
- * This creates the named directory with the given @mode, creating
- * any necessary intermediate directories (with the same @mode).
- *
- * Return value: 0 on success, -1 on error, in which case errno will
- * be set as for mkdir(2).
- **/
-static int
-e_mkdir_hier(const char *path, mode_t mode)
+static char *
+sanitize_path (const char *path)
 {
-	char *copy, *p;
+	gchar **comps;
+	char *new_path = NULL;
 
-	if (path[0] == '/') {
-		p = copy = g_strdup (path);
-	} else {
-		gchar *current_dir = g_get_current_dir();
-		p = copy = g_build_filename (current_dir, path, NULL);
-	}
+	if (!path)
+		return g_strdup("");	/* ??? or NULL? */
 
-	do {
-		p = strchr (p + 1, '/');
-		if (p)
-			*p = '\0';
-		if (access (copy, F_OK) == -1) {
-			if (mkdir (copy, mode) == -1) {
-				g_free (copy);
-				return -1;
-			}
-		}
-		if (p)
-			*p = '/';
-	} while (p);
+	comps = g_strsplit (path, ";", 2);
+	if (comps[1])
+		new_path = g_strdup_printf ("%s%s", comps[0], comps[1]);
+	else if (comps[0])
+		new_path = g_strdup (comps[0]);
 
-	g_free (copy);
-	return 0;
+	g_strfreev (comps);
+	return new_path;	
 }
 
+
 /**
  * e_folder_exchange_new:
  * @hier: the #ExchangeHierarchy containing the new folder
@@ -180,11 +166,9 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 	efe->priv->hier = hier;
 	g_object_ref (hier);
 	efe->priv->internal_uri = g_strdup (internal_uri);
-	efe->priv->path = e2k_uri_path (e_folder_get_physical_uri (ef));
+	efe->priv->path = sanitize_path (e2k_uri_path (e_folder_get_physical_uri (ef)));
 	efe->priv->outlook_class = g_strdup (outlook_class);
 
-#if 0
-SURF :
 	/* Add ESources */
 	if (hier->type == EXCHANGE_HIERARCHY_PERSONAL || 
 	    hier->type == EXCHANGE_HIERARCHY_FAVORITES) {
@@ -211,7 +195,6 @@ SURF :
 				     	    physical_uri);
 		}
 	}
-#endif	
 	return ef;
 }
 
@@ -425,7 +408,7 @@ e_folder_exchange_get_storage_file (EFolder *folder, const char *filename)
 		efe->priv->storage_dir = e_path_to_physical (
 			efe->priv->hier->account->storage_dir,
 			efe->priv->path);
-		e_mkdir_hier (efe->priv->storage_dir, 0755);
+		e_util_mkdir_hier (efe->priv->storage_dir, 0755);
 	}
 
 	path = g_build_filename (efe->priv->storage_dir, filename, NULL);
@@ -481,31 +464,16 @@ e_folder_exchange_save_to_file (EFolder *folder, const char *filename)
 	if (permanent_uri)
 		xmlNewChild (root, NULL, "permanent_uri", permanent_uri);
 
-	status = xmlSaveFile (filename, doc);
-	xmlFreeDoc (doc);
+	status = e_xml_save_file (filename, doc);
+
 	if (status < 0)
-		unlink (filename);
+		g_unlink (filename);
+
+	xmlFreeDoc (doc);
 
 	return status == 0;
 }
 
-/* Taken this from the old GAL code */
-static xmlNode *
-e_xml_get_child_by_name (const xmlNode *parent, const xmlChar *child_name)
-{
-	xmlNode *child;
-
-	g_return_val_if_fail (parent != NULL, NULL);
-	g_return_val_if_fail (child_name != NULL, NULL);
-
-	for (child = parent->xmlChildrenNode; child != NULL; child = child->next) {
-		if (xmlStrcmp (child->name, child_name) == 0) {
-			return child;
-		}	
-	}
-	return NULL;
-}
-
 /**
  * e_folder_exchange_new_from_file:
  * @hier: the hierarchy to create the folder under
@@ -527,7 +495,8 @@ e_folder_exchange_new_from_file (ExchangeHierarchy *hier, const char *filename)
 	char *permanent_uri = NULL;
 	char *folder_size = NULL;
 
-	doc = xmlParseFile (filename);
+	doc = e_xml_parse_file (filename);
+
 	if (!doc)
 		return NULL;
 
@@ -932,8 +901,6 @@ e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 	const char *folder_type, *physical_uri;
 
 	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), E2K_HTTP_MALFORMED);
-#if 0
-SURF :
 	/* remove ESources */
 	hier = e_folder_exchange_get_hierarchy (folder); 
 
@@ -961,7 +928,7 @@ SURF :
 					       physical_uri);
 		}
 	}
-#endif
+
 	return e2k_context_delete (E_FOLDER_EXCHANGE_CONTEXT (folder), op,
 				   E_FOLDER_EXCHANGE_URI (folder));
 }
