@@ -3,7 +3,7 @@
 /* Copyright (C) 2001-2004 Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -30,7 +30,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-static gboolean is_offline ();
+static gboolean is_offline (void);
 
 void
 add_folder_esource (ExchangeAccount *account, 
@@ -145,6 +145,7 @@ add_folder_esource (ExchangeAccount *account,
 				if (mode == OFFLINE_MODE) 
 					e_source_set_property (source, "offline_sync", "1");
 			}
+			e_source_list_sync (source_list, NULL);
 		}
 	}
 
@@ -307,7 +308,7 @@ remove_folder_esource (ExchangeAccount *account,
 }
 
 static gboolean 
-is_offline () 
+is_offline (void) 
 {
 	GConfClient *client;
 	GConfValue *value;
