@@ -3,7 +3,7 @@
 /* Copyright (C) 2002-2004 Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -26,10 +26,21 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include <glib.h>
+#include <glib/gstdio.h>
+
+#include "libedataserverui/e-passwords.h"
+#include "libedataserver/e-source-list.h"
+
 #include "exchange-hierarchy-webdav.h"
 #include "exchange-account.h"
 #include "e-folder-exchange.h"
 #include "e2k-context.h"
+#include "e2k-path.h"
 #include "e2k-propnames.h"
 #include "e2k-restriction.h"
 #include "e2k-uri.h"
@@ -37,14 +48,6 @@
 #include "exchange-folder-size.h"
 #include "exchange-esource.h"
 
-#include <libedataserverui/e-passwords.h>
-#include "e2k-path.h"
-#include <libedataserver/e-source-list.h>
-
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
 struct _ExchangeHierarchyWebDAVPrivate {
 	GHashTable *folders_by_internal_path;
 	gboolean deep_searchable;
@@ -63,7 +66,7 @@ static gboolean is_empty (ExchangeHierarchy *hier);
 static void rescan (ExchangeHierarchy *hier);
 static ExchangeAccountFolderResult scan_subtree  (ExchangeHierarchy *hier,
 						  EFolder *folder,
-						  gboolean offline);
+						  int mode);
 static ExchangeAccountFolderResult create_folder (ExchangeHierarchy *hier,
 						  EFolder *parent,
 						  const char *name,
@@ -206,7 +209,7 @@ hierarchy_removed_folder (ExchangeHierarchy *hier, EFolder *folder,
 			     (char *)e2k_uri_path (internal_uri));
 
 	mf_path = e_folder_exchange_get_storage_file (folder, "connector-metadata.xml");
-	unlink (mf_path);
+	g_unlink (mf_path);
 	g_free (mf_path);
 
 	e_path_rmdir (hier->account->storage_dir,
@@ -383,10 +386,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 	char *permanent_url = NULL, *physical_uri, *source_parent;
 	const char *folder_type = NULL, *source_folder_name;
 	ExchangeAccountFolderResult ret_code;
-	int offline;
+	int mode;
 
-	exchange_account_is_offline (hier->account, &offline);
-        if (offline != ONLINE_MODE)
+	exchange_account_is_offline (hier->account, &mode);
+        if (mode != ONLINE_MODE)
                 return EXCHANGE_ACCOUNT_FOLDER_OFFLINE;
 
 	if (source == hier->toplevel)
@@ -409,7 +412,7 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 		if (remove_source)
 			exchange_hierarchy_removed_folder (hier, source);
 		exchange_hierarchy_new_folder (hier, dest);
-		scan_subtree (hier, dest, (offline == OFFLINE_MODE));
+		scan_subtree (hier, dest, mode);
 		physical_uri = (char *) e_folder_get_physical_uri (source);
 		g_object_unref (dest);
 		ret_code = EXCHANGE_ACCOUNT_FOLDER_OK;
@@ -503,12 +506,12 @@ rescan (ExchangeHierarchy *hier)
 	E2kResultIter *iter;
 	E2kResult *result;
 	EFolder *folder;
-	int unread, offline;
+	int unread, mode;
 	gboolean personal = ( hier->type == EXCHANGE_HIERARCHY_PERSONAL );
 	gdouble fsize_d;
 
-	exchange_account_is_offline (hier->account, &offline);
-	if ( (offline != ONLINE_MODE) ||
+	exchange_account_is_offline (hier->account, &mode);
+	if ( (mode != ONLINE_MODE) ||
 		hier->type == EXCHANGE_HIERARCHY_PUBLIC)
 		return;
 
@@ -677,7 +680,7 @@ static const char *folder_props[] = {
 static const int n_folder_props = sizeof (folder_props) / sizeof (folder_props[0]);
 
 static ExchangeAccountFolderResult
-scan_subtree (ExchangeHierarchy *hier, EFolder *parent, gboolean offline)
+scan_subtree (ExchangeHierarchy *hier, EFolder *parent, int mode)
 {
 	static E2kRestriction *folders_rn;
 	ExchangeHierarchyWebDAV *hwd = EXCHANGE_HIERARCHY_WEBDAV (hier);
@@ -692,7 +695,7 @@ scan_subtree (ExchangeHierarchy *hier, EFolder *parent, gboolean offline)
 	const char *name, *folder_size, *deleted_items_uri, *int_uri;
 	gboolean personal = ( EXCHANGE_HIERARCHY (hwd)->type == EXCHANGE_HIERARCHY_PERSONAL );
 
-	if (offline) {
+	if (mode == OFFLINE_MODE) {
 		folders = g_ptr_array_new ();
 		exchange_hierarchy_webdav_offline_scan_subtree (EXCHANGE_HIERARCHY (hier), add_folders, folders);
 		for (i = 0; i <folders->len; i++) {
@@ -753,7 +756,7 @@ scan_subtree (ExchangeHierarchy *hier, EFolder *parent, gboolean offline)
 		int_uri = e_folder_exchange_get_internal_uri (folder);
 		if (int_uri && !strcmp (int_uri, deleted_items_uri))
 			continue;
-		scan_subtree (hier, folder, offline);
+		scan_subtree (hier, folder, mode);
 	}
 
 	return exchange_hierarchy_webdav_status_to_folder_result (status);
@@ -777,7 +780,7 @@ scan_offline_cb (const char *physical_path, const char *path, gpointer data)
 	mf_name = g_build_filename (physical_path, "connector-metadata.xml", NULL);
 	folder = e_folder_exchange_new_from_file (sod->hier, mf_name);
 	if (!folder) {
-		unlink (mf_name);
+		g_unlink (mf_name);
 		g_free (mf_name);
 		if (!sod->badpaths)
 			sod->badpaths = g_ptr_array_new ();
