@@ -7,15 +7,15 @@
  * Copyright 2003, Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU General Public 
+ * modify it under the terms of version 2 of the GNU Lesser General Public 
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -125,6 +125,7 @@ typedef struct {
 	char *id ;
 	char *name ;
 	char *item_reference;
+	char *contentid;
 	char *contentType ;
 	int size ;
 	char *date ;
@@ -169,6 +170,8 @@ const char *e_gw_item_get_id (EGwItem *item);
 void        e_gw_item_set_id (EGwItem *item, const char *new_id);
 char       *e_gw_item_get_creation_date (EGwItem *item);
 void        e_gw_item_set_creation_date (EGwItem *item, const char *new_date);
+char       *e_gw_item_get_delivered_date (EGwItem *item);
+void        e_gw_item_set_delivered_date (EGwItem *item, const char *new_date);
 char       *e_gw_item_get_start_date (EGwItem *item);
 void        e_gw_item_set_start_date (EGwItem *item, const char *new_date);
 char       *e_gw_item_get_completed_date (EGwItem *item);
